from threading import Condition


def zippingFilter(a, b):
    return [ x for x in enumerate(zip(a, b)) if x[1][0] == x[1][1] ]

class NotInitializedException(Exception): pass

class ProcessImage(Condition):
    def __init__(self):
        super().__init__()
        self.initialized = False

    def init(self, numCoils, numDiscreteInputs, numAnalogInputs):
        self.coils = []
        self.shadowCoils = [ None ] * numCoils

        self.discreteInputs = []
        self.shadowDiscreteInputs = [ None ] * numDiscreteInputs

        self.analogInputs = []
        self.shadowAnalogInputs = [ None ] * (numAnalogInputs // 8)

        self.initialized = True
    
    def isInitialized(self):
        return self.initialized

    def hasPendingInputChanges(self):
        return (self.discreteInputs != self.shadowDiscreteInputs) or (self.analogInputs != self.shadowAnalogInputs)

    def hasPendingOutputChanges(self):
        return self.shadowCoils != self.coils

    def setDiscreteInputs(self, discreteInputs):
        if not self.initialized:
            raise NotInitializedException
        self.discreteInputs = discreteInputs

    def getChangedDiscreteInputs(self):
        if not self.initialized:
            raise NotInitializedException
        print("D1: {}".format(self.discreteInputs))
        print("D2: {}".format(self.shadowDiscreteInputs))
        changedDiscreteInputs = zippingFilter(self.discreteInputs, self.shadowDiscreteInputs)
        print("D3: {}".format(changedDiscreteInputs))
        self.shadowDiscreteInputs = self.discreteInputs
        print("D4: {}".format(self.shadowDiscreteInputs))
        return changedDiscreteInputs
    
    def getDiscreteInputs(self):
        if not self.initialized:
            raise NotInitializedException
        self.shadowDiscreteInputs = self.discreteInputs
        return self.discreteInputs

    def setCoils(self, coils):
        if not self.initialized:
            raise NotInitializedException
        self.coils = coils

    def getChangedCoils(self):
        if not self.initialized:
            raise NotInitializedException
        changedCoils = zippingFilter(self.coils, self.shadowCoils)
        self.shadowCoils = self.coils
        return changedCoils
    
    def getCoils(self):
        if not self.initialized:
            raise NotInitializedException
        self.shadowCoils = self.coils
        return self.coils

    def setAnalogsInputs(self, analogInputs):
        if not self.initialized:
            raise NotInitializedException
        self.analogInputs = analogInputs

    def getChangedAnalogsInputs(self):
        if not self.initialized:
            raise NotInitializedException
        changedAnalogInputs = zippingFilter(self.analogInputs, self.shadowAnalogInputs)
        self.shadowAnalogInputs = self.analogInputs
        return changedAnalogInputs
    
    def getAnalogsInputs(self):
        if not self.initialized:
            raise NotInitializedException
        self.shadowAnalogInputs = self.analogInputs
        return self.analogInputs


    

    

